SystemOrganization addCategory: #'DAO-Satellite'!

TestCase subclass: #CommandInterpreterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!CommandInterpreterTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:18'!
commandInterpreter

	| commandInterpreter subsystems thermalSubsystem |
	
	subsystems := OrderedCollection new.

	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	subsystems add: thermalSubsystem.

	commandInterpreter := CommandInterpreter newWithSubsystems: subsystems.
	
	^ commandInterpreter.! !

!CommandInterpreterTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:14'!
testWhenReceivesSubsystemIdTheCorrespondingSubsytemIsResolved

	| commandInterpreter subsystem |

	commandInterpreter := self commandInterpreter.
	
	subsystem := commandInterpreter resolveSubsystem: 3.
	
	self assert: (subsystem name) equals: 'THERMAL'.
! !

TestCase subclass: #SatelliteSubsystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:46'!
testWhenCreatingASubsystemItCanAnswerBasicCommands

	| subsystem |

	"setup"
	subsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.

	"verify"	
	self assert: (subsystem understands: 'name').
	self assert: (subsystem understands: 'telemetry').
	self assert: (subsystem understands: 'short_telemetry').
! !

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:15'!
testWhenCreatingASubsystemItDoesntUnderstandUnknwonCommand

	| subsystem |

	"setup"
	subsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.

	"verify"	
	self deny: (subsystem understands: 'fruit').
! !

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:15'!
testWhenCreatingASubsystemTheIdAndNameIsSet

	| subsystem |

	"setup"
	subsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.

	"verify"	
	self assert: subsystem id equals: 1.
	self assert: subsystem name equals: 'MEMORY'.
! !

TestCase subclass: #SatelliteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 12:46'!
satelliteWithThermalAndMemorySubsystems
	| satellite subsystems thermalSubsystem memorySubsystem |

	subsystems := Set new.
	
	memorySubsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.
	memorySubsystem setAttribute: 'free' withValue: 100.
	subsystems add: memorySubsystem.
	
	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem setAttribute: 'battery_temp' withValue: 10.
	thermalSubsystem setAttribute: 'radio_temp' withValue: 20.
	subsystems add: thermalSubsystem.

	satellite := Satellite newWithSubsystems: subsystems.
	^ satellite! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 12:46'!
testGivenASatelliteWhenEmittingAShortBeaconItReturnsAShortBeacon

	| satellite subsystems thermalSubsystem expectedBeacon beacon |

	"setup"
	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem setAttribute: 'battery_temp' withValue: '10'.
	thermalSubsystem setAttribute: 'radio_temp' withValue: '20'.
	subsystems := Set new.
	subsystems add: thermalSubsystem.
	satellite := Satellite newWithSubsystems: subsystems.
	expectedBeacon := '{"short_telemetry":[{"subsystem_id":"3","battery_temp":"10"}]}'.

	"exercise"	
	beacon := satellite emitShortBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 12:46'!
testGivenASatelliteWithThermalSubsystemOnlyWhenEmittingABeaconTheBeaconContainsOnlyThermalTelemetry

	| satellite subsystems thermalSubsystem expectedBeacon beacon |

	"setup"
	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem setAttribute: 'battery_temp' withValue: '10'.
	thermalSubsystem setAttribute: 'radio_temp' withValue: '20'.
	subsystems := Set new.
	subsystems add: thermalSubsystem.
	satellite := Satellite newWithSubsystems: subsystems.
	expectedBeacon := '{"telemetry":[{"subsystem_id":"3","battery_temp":"10","radio_temp":"20"}]}'.

	"exercise"	
	beacon := satellite emitBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:26'!
testGivenASatelliteWithTwoSubsystemsWhenEmittingABeaconTheBeaconContainsTwoTelemetryData

	| satellite expectedBeacon beacon |

	"setup"
	satellite := self satelliteWithThermalAndMemorySubsystems.

	expectedBeacon := '{"telemetry":[{"subsystem_id":"1","free":"100"},{"subsystem_id":"3","battery_temp":"10","radio_temp":"20"}]}'.

	"exercise"	
	beacon := satellite emitBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 13:44'!
testWhenSendingCommandToSubsystemIndexedByIdTheSpecificSubsystemResponds
	| satellite response echoResponse |
	
	"setup"
	satellite := self satelliteWithThermalAndMemorySubsystems.
	echoResponse := MessageCommandResponse newWithMessage: 'THERMAL'.

	"exercise"
	"subsystem 3 is THERMAL"
	response := satellite execute: 'name' onSubsystem: 3.
	
	"verify"
	self assert: response equals: echoResponse.! !

!SatelliteTest methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 13:44'!
testWhenSendingSetMatrixKeyCommandToThermalSubsystemTheResponseShouldBeVoid
	| satellite response voidResponse |
	
	"setup"
	satellite := self satelliteWithThermalAndMemorySubsystems.
	voidResponse := VoidCommandResponse new.

	"exercise"
	"subsystem 3 is THERMAL"
	response := satellite execute: 'setMatrixKey' onSubsystem: 3 withParameter: 1.
	
	"verify"
	self assert: response equals: voidResponse.
! !

Object subclass: #CommandInterpreter
	instanceVariableNames: 'subsystems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!CommandInterpreter class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:14'!
newWithSubsystems: theSubsystems
	^ self new initializeWithSubystems: theSubsystems.! !

!CommandInterpreter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:13'!
initializeWithSubystems: theSubsystems
	subsystems := Dictionary new.
	theSubsystems do: [ :subsystem | subsystems at: (subsystem id) put: subsystem].
	! !

!CommandInterpreter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:10'!
resolveSubsystem: aKey 
	^ subsystems at: aKey.! !

Object subclass: #CommandResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

CommandResponse subclass: #MessageCommandResponse
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MessageCommandResponse class methodsFor: 'class initialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:06'!
newWithMessage: aMessage 
	self new initializeWithMessage: aMessage.! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:01'!
= anObject
	^ message = anObject message
! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:02'!
initialize
	message = ''
! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:06'!
initializeWithMessage: aMessage 
	message := aMessage.! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:02'!
message
	^ message
! !

CommandResponse subclass: #VoidCommandResponse
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!VoidCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:58'!
= anotherResponse
	"TODO reemplazar con una sola instancia global de VoidCommandResponse"
	^ code = anotherResponse code ! !

!VoidCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:58'!
code
	^ code! !

!VoidCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:56'!
initialize
	code = -1
! !

Object subclass: #Satellite
	instanceVariableNames: 'subsystems commandInterpreter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!Satellite class methodsFor: 'instance creation' stamp: 'EmilianoMenendez 9/7/2013 16:27'!
newWithSubsystems: theSubsystems 
	^ self new initalizeWithSubsystems: theSubsystems! !

!Satellite methodsFor: 'signaling' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:48'!
emitBeacon
	| result outputStream telemetryDataOrderedBySubsystemId |
	
	outputStream := WriteStream on: String new.
	
	outputStream nextPutAll: '{"telemetry":['.
	
	telemetryDataOrderedBySubsystemId := OrderedIdentityDictionary new.
	
	subsystems do: [ :subsystem | telemetryDataOrderedBySubsystemId at: (subsystem id) put: subsystem telemetry].
	
	telemetryDataOrderedBySubsystemId do: [ :telemetry | outputStream nextPutAll: telemetry. outputStream nextPutAll: ',' ].
	
	result := outputStream contents.
	
	result := result truncateTo: ((result size) - 1).
	
	result := result , ']}'.
	
	^ result.
! !

!Satellite methodsFor: 'signaling' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 21:06'!
emitShortBeacon
	| result outputStream telemetryDataOrderedBySubsystemId |
	
	outputStream := WriteStream on: String new.
	
	outputStream nextPutAll: '{"short_telemetry":['.
	
	telemetryDataOrderedBySubsystemId := OrderedIdentityDictionary new.
	
	subsystems do: [ :subsystem | telemetryDataOrderedBySubsystemId at: (subsystem id) put: subsystem shortTelemetry].
	
	telemetryDataOrderedBySubsystemId do: [ :telemetry | outputStream nextPutAll: telemetry. outputStream nextPutAll: ',' ].
	
	result := outputStream contents.
	
	result := result truncateTo: ((result size) - 1).
	
	result := result , ']}'.
	
	^ result.
! !

!Satellite methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:33'!
execute: commandName onSubsystem: subsystemId
	| subsystem command response |
	
	subsystem := commandInterpreter resolveSubsystem: subsystemId.
	
	command := subsystem commands at: commandName.
	
	response := command executeOn: subsystem.
	
	^ response
! !

!Satellite methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:33'!
execute: commandName onSubsystem: subsystemId withParameter: parameter
	| subsystem command response |
	
	subsystem := commandInterpreter resolveSubsystem: subsystemId.
	
	command := subsystem commands at: commandName.
	
	response := command executeOn: subsystem withParameter: parameter.
	
	^ response.! !

!Satellite methodsFor: 'initialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:32'!
initalizeWithSubsystems: theSubsystems

	subsystems := theSubsystems.
	
	commandInterpreter := CommandInterpreter newWithSubsystems: subsystems.


! !

!Satellite methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:59'!
subsystems
	
	^ subsystems
! !

Object subclass: #SatelliteCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SatelliteCommand subclass: #MatrixKeyCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MatrixKeyCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:52'!
executeOn: target withParameter: parameter
	target setMatrixKey: parameter.
	
	^ VoidCommandResponse new.
	! !

SatelliteCommand subclass: #NameCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!NameCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:54'!
executeOn: target
	^ MessageCommandResponse newWithMessage: (target name)! !

!SatelliteCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:31'!
executeOn: target
	^ self subclassResponsibility.! !

!SatelliteCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 19:45'!
executeOn: target withParameter: parameter
	^ self subclassResponsibility.! !

SatelliteCommand subclass: #ShortTelemetryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SatelliteCommand subclass: #TelemetryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

Object subclass: #SatelliteSubsystem
	instanceVariableNames: 'id data telemetryKeys shortTelemetryKeys name commands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SatelliteSubsystem subclass: #MemorySatelliteSubsystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MemorySatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:00'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'free'.
	
	shortTelemetryKeys := Set new.
	shortTelemetryKeys add: 'free'.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys.
	
	^ subsystem.! !

!SatelliteSubsystem class methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:28'!
basicCommands
	| commands |
	
	commands := Dictionary new.

	commands at: 'name' put: NameCommand new.
	commands at: 'telemetry' put: TelemetryCommand new.
	commands at: 'short_telemetry' put: ShortTelemetryCommand new.

	^ commands! !

!SatelliteSubsystem class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 18:58'!
newWithId: theId andName: theName
	^ self new initializeWithId: theId andName: theName andTelemetryKeys: Set new andShortTelemetryKeys: Set new! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 18:27'!
commands
	^ commands! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:10'!
id
	^ id! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:26'!
initializeWithId: theId andName: theName andTelemetryKeys: theTelemetryKeys andShortTelemetryKeys: theShortTelemetryKeys
	id := theId.
	name := theName.
	commands := SatelliteSubsystem basicCommands.
	telemetryKeys := theTelemetryKeys.
	shortTelemetryKeys := theShortTelemetryKeys.
	data := Dictionary new.! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 19:37'!
initializeWithId: theId andName: theName andTelemetryKeys: theTelemetryKeys andShortTelemetryKeys: theShortTelemetryKeys andCommands: theCommands
	id := theId.
	name := theName.
	commands := SatelliteSubsystem basicCommands.
	commands add: theCommands.
	telemetryKeys := theTelemetryKeys.
	shortTelemetryKeys := theShortTelemetryKeys.
	data := Dictionary new.! !

!SatelliteSubsystem methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:11'!
name
	^ name! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 12:46'!
setAttribute: aKey withValue: aValue 
	data at: aKey put: aValue! !

!SatelliteSubsystem methodsFor: 'command' stamp: 'EmilianoMenendez 9/9/2013 10:26'!
shortTelemetry
	| result temp |
	
	temp := WriteStream on: String new.
	
	temp nextPutAll: '{'.
	
	"add subsytem_id"
	temp nextPutAll: '"subsystem_id":"'.
	temp nextPutAll: id asString.
	temp nextPutAll: '",'.
	
	"build key:value,key:value,,, string "
	data associationsDo: [ :entry | (shortTelemetryKeys includes: (entry key)) ifTrue: [temp nextPutAll: '"'. temp nextPutAll: (entry key asString) . temp nextPutAll: '":"'. temp nextPutAll: (entry value asString). temp nextPutAll: '",'.]].
	
	"send stream to string"
	result := temp contents.
	
	"remove last ,"
	result := result truncateTo: ((result size) - 1).
	
	result := result , '}'.
	
	^ result
! !

!SatelliteSubsystem methodsFor: 'command' stamp: 'EmilianoMenendez 9/8/2013 20:40'!
telemetry
	| result temp |
	
	temp := WriteStream on: String new.
	
	temp nextPutAll: '{'.
	
	"add subsytem_id"
	temp nextPutAll: '"subsystem_id":"'.
	temp nextPutAll: id asString.
	temp nextPutAll: '",'.
	
	"build key:value,key:value,,, string "
	data associationsDo: [ :entry | temp nextPutAll: '"'. temp nextPutAll: (entry key asString) . temp nextPutAll: '":"'. temp nextPutAll: (entry value asString). temp nextPutAll: '",'.].
	
	"send stream to string"
	result := temp contents.
	
	"remove last ,"
	result := result truncateTo: ((result size) - 1).
	
	result := result , '}'.
	
	^ result
! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:50'!
understands: aCommandName 
	^ (commands includesKey: aCommandName).! !

SatelliteSubsystem subclass: #ThermalSatelliteSubsystem
	instanceVariableNames: 'matrixKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ThermalSatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 19:45'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys matrixKeyCommand matrixKeyCommandEntry |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'battery_temp'.
	telemetryKeys add: 'radio_temp'.
	
	shortTelemetryKeys := Set new.
	shortTelemetryKeys add: 'battery_temp'.
	
	matrixKeyCommand := MatrixKeyCommand new.
	matrixKeyCommandEntry := Association key: 'setMatrixKey' value: matrixKeyCommand.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys andCommands: matrixKeyCommandEntry.
	
	^ subsystem.! !

!ThermalSatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 19:46'!
setMatrixKey: theKey
	matrixKey := theKey! !
